from mahabharata.models import MahabharataPoems


POEMS = MahabharataPoems.parse_obj(
    [
        {
            "quote": "Time creates all things and time destroys them all. Time burns all creatures and time again extinguishes that fire",
            "authors": "Anukramanika Parva, Adi Parva",
        },
        {
            "quote": "What is a greater happiness to a father than what the father feels when his son, running to him, clasps him with his tiny little arms, though his body is full of dust and dirt.",
            "authors": "Sambhava Parva, Adi Parva",
        },
        {
            "quote": "Friendship never remains in the world in anyone’s heart without being worn out, Time wears it out, anger destroys it. The poor cannot be the friend of the rich, the unlearned cannot be the friend of the learned, the coward cannot be the friend of the brave.",
            "authors": "Drupada to Drona, Sambhava Parva, Adi Parva",
        },
        {
            "quote": "Men of immature understanding begin an act without having an eye to what may happen in future.",
            "authors": "Rajshuyarambha Parva, Sabha Parva",
        },
        {
            "quote": "Whatever actions are performed by a man under whatever circumstances, he gets the fruits of those actions under whatever circumstances they may be performed.",
            "authors": "Krishna, Jarasandhabadha Parva Sabha Parva",
        },
        {
            "quote": "Men experience happiness and misery by turn. No man ever enjoys unmixed happiness. A wise man, possessing high wisdom, knowing that life has its ups and downs, is neither filled with joy nor with grief. When happiness comes, one should enjoy it, and when misery comes one should bear it",
            "authors": "Vrihi Drounika Parva, Aranya Parva",
        },
        {
            "quote": "Pleasant looks, cheerful heart and sweet words are due to a guest. Rising up, the host should advance towards the guest; he should offer him a seat, and duly worship him. This is the eternal Dharma.",
            "authors": "Aranyaka Parva, Aranya Parva",
        },
        {
            "quote": "Anger is in this world, the root of the destruction of mankind, The angry man commits a sin; the angry man murders his preceptor; the angry man insults his ciders with harsh words. The angry man cannot distinguish what should be and should not be said by him. there is nothing which cannot be said or done by an angry man.",
            "authors": "Yudhishthira, Arjunabhigamana Parva, Aranya Parva",
        },
        {
            "quote": "A wise man should never contract friendship with the wife of the king nor with other attendants of his, nor with those whom he despises and who are hostile to him.",
            "authors": "Pandava Pravesha Parva, Virata Parva",
        },
    ]
)



import click
from mahabharata.services.poet_service.query import get_all_verses_by_author, all_verses


@click.command()
@click.option("--author", "-u", help="Query poem by this author", type=str)
@click.option("--all", "-a", help="list all in scope, overides limit", is_flag=True)
@click.option("--limit", "-l", help="Limit poems to 'N', where 0 < N <= 10", default=1)
@click.option("--collabs", "-c", help="Show collaborators with poem", is_flag=True)
def verses(author: str, all: bool, limit: int, collabs: bool) -> None:
    """Query Mahabharata poems, the default behaviour will give you all verses in list with there authors."""

    def get_quote(iterable, all):
        for index, verse in enumerate(iterable):
            if index == limit and not all:
                return

            click.echo(verse.quote)
            click.echo(f"by: {author or ', '.join(verse.authors).strip(', ')}")
            if collabs and author:
                collaborators = {
                    co_author for co_author in verse.authors if co_author != author
                }
                click.echo(
                    f"collaborator{'s' if len(collaborators) > 1 else ''}: {', '.join(collaborators).strip(', ')}"
                )
            click.echo()

    if not author:
        get_quote(all_verses(), all=(limit == 1))

    else:
        get_quote(get_all_verses_by_author(author=author), all=all)



from typing import Set, Optional, Tuple

from mahabharata.models.poem import Verse
from . import POEMS


def list_all_authors() -> Set[str]:
    return {author for verse in POEMS for author in verse.authors}


def list_all_collaborators(author: str) -> Set[str]:
    return {
        collaborator
        for verse in POEMS
        for collaborator in verse.authors
        if author in verse.authors and collaborator != author
    }


def get_verse_from_author(author: str) -> Optional[Verse]:
    try:
        return POEMS.verse_by_speaker(author)
    except:
        return None


def all_verses() -> Tuple[Verse, ...]:
    return POEMS


def get_all_verses_by_author(author: str) -> Tuple[Verse, ...]:
    return tuple(verse for verse in POEMS if author in verse.authors)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mahabharata',
 'mahabharata.models',
 'mahabharata.services',
 'mahabharata.services.cli_service',
 'mahabharata.services.poet_service']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0']

extras_require = \
{':python_version >= "3.7" and python_version < "4.0"': ['pydantic>=1.8.2,<2.0.0'],
 'api': ['gunicorn>=20.1.0,<21.0.0',
         'uvicorn>=0.15.0,<0.16.0',
         'fastapi>=0.68.0,<0.69.0']}

setup_kwargs = {
    'name': 'mahabharata',
    'version': '0.2.0',
    'description': '',
    'long_description': None,
    'author': 'Sharlon Regales',
    'author_email': 'sharlon.regales@tno.nl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Tuple, Iterator

from pydantic import BaseModel, validator


class _BaseModelConfig(BaseModel):
    class Config:
        validate_all = True
        validate_assignment = True
        extra = "forbid"
        allow_mutation = False


class Verse(_BaseModelConfig):
    quote: str
    authors: Tuple[str, ...]

    @validator("authors", pre=True)
    def split_str(cls, value):
        if isinstance(value, str):
            return tuple(map(lambda author: author.strip(), value.split(",")))
        return value


class MahabharataPoems(_BaseModelConfig):
    __root__: Tuple[Verse, ...]

    def verse_by_speaker(self, author) -> Verse:
        return next((verse for verse in self if author in verse.authors), None)

    def __iter__(self) -> Iterator[Verse]:
        return iter(self.__root__)

    def __getitem__(self, item) -> Verse:
        return self.__root__[item]

    def __len__(self) -> int:
        return len(self.__root__)
